@@ -92,7 +92,7 @@ If nil, only read articles will be expired."
 (defvar gnus-agent-spam-hashtb nil)
 (defvar gnus-agent-file-name nil)
 (defvar gnus-agent-send-mail-function nil)
-(defvar gnus-agent-article-file-coding-system 'no-conversion)
+(defvar gnus-agent-file-coding-system 'no-conversion)
 
 (defconst gnus-agent-scoreable-headers
   (list
@@ -115,6 +115,8 @@ for download via the Agent.")
   (gnus-category-read)
   (setq gnus-agent-overview-buffer
 	(gnus-get-buffer-create " *Gnus agent overview*"))
+  (with-current-buffer gnus-agent-overview-buffer
+    (mm-enable-multibyte))
   (add-hook 'gnus-group-mode-hook 'gnus-agent-mode)
   (add-hook 'gnus-summary-mode-hook 'gnus-agent-mode)
   (add-hook 'gnus-server-mode-hook 'gnus-agent-mode))
@@ -341,7 +343,7 @@ agent minor mode in all Gnus buffers."
      (concat "^" (regexp-quote mail-header-separator) "\n"))
     (replace-match "\n")
     (gnus-agent-insert-meta-information 'mail)
-    (gnus-request-accept-article "nndraft:queue")))
+    (gnus-request-accept-article "nndraft:queue" nil t t)))
 
 (defun gnus-agent-insert-meta-information (type &optional method)
   "Insert meta-information into the message that says how it's to be posted.
@@ -524,7 +526,7 @@ the actual number of articles toggled is returned."
     (let* ((gnus-command-method method)
 	   (file (gnus-agent-lib-file "active")))
       (gnus-make-directory (file-name-directory file))
-      (let ((coding-system-for-write gnus-agent-article-file-coding-system))
+      (let ((coding-system-for-write gnus-agent-file-coding-system))
 	(write-region (point-min) (point-max) file nil 'silent))
       (when (file-exists-p (gnus-agent-lib-file "groups"))
 	(delete-file (gnus-agent-lib-file "groups"))))))
@@ -533,9 +535,10 @@ the actual number of articles toggled is returned."
   (let* ((gnus-command-method method)
 	 (file (gnus-agent-lib-file "groups")))
     (gnus-make-directory (file-name-directory file))
-    (write-region (point-min) (point-max) file nil 'silent))
+    (let ((coding-system-for-write gnus-agent-file-coding-system))
+      (write-region (point-min) (point-max) file nil 'silent))
     (when (file-exists-p (gnus-agent-lib-file "active"))
-      (delete-file (gnus-agent-lib-file "active"))))
+      (delete-file (gnus-agent-lib-file "active")))))
 
 (defun gnus-agent-save-group-info (method group active)
   (when (gnus-agent-method-p method)
@@ -554,7 +557,7 @@ the actual number of articles toggled is returned."
 		     (concat "^" (regexp-quote group) " ") nil t)
 		(gnus-delete-line))
 	      (insert group " " (number-to-string (cdr active)) " "
-		      (number-to-string (car active)) "\n"))
+		      (number-to-string (car active)) " y\n"))
 	  (when (re-search-forward (concat (regexp-quote group) " ") nil t)
 	    (gnus-delete-line))
 	  (insert-buffer-substring nntp-server-buffer))))))
@@ -604,8 +607,9 @@ the actual number of articles toggled is returned."
   (save-excursion
     (set-buffer gnus-agent-current-history)
     (gnus-make-directory (file-name-directory gnus-agent-file-name))
-    (write-region (1+ (point-min)) (point-max)
-		  gnus-agent-file-name nil 'silent)))
+    (let ((coding-system-for-write gnus-agent-file-coding-system))
+      (write-region (1+ (point-min)) (point-max)
+		    gnus-agent-file-name nil 'silent))))
 
 (defun gnus-agent-close-history ()
   (when (gnus-buffer-live-p gnus-agent-current-history)
@@ -703,7 +707,7 @@ the actual number of articles toggled is returned."
 		(setq id "No-Message-ID-in-article")
 	      (setq id (buffer-substring (match-beginning 1) (match-end 1))))
 	    (let ((coding-system-for-write
-		   gnus-agent-article-file-coding-system))
+		   gnus-agent-file-coding-system))
 	      (write-region (point-min) (point-max)
 			    (concat dir (number-to-string (caar pos)))
 			    nil 'silent))
@@ -746,10 +750,12 @@ the actual number of articles toggled is returned."
   (save-excursion
     (while gnus-agent-buffer-alist
       (set-buffer (cdar gnus-agent-buffer-alist))
-      (write-region (point-min) (point-max)
-		    (gnus-agent-article-name ".overview"
-					     (caar gnus-agent-buffer-alist))
-		     nil 'silent)
+      (let ((coding-system-for-write
+	     gnus-agent-file-coding-system))
+	(write-region (point-min) (point-max)
+		      (gnus-agent-article-name ".overview"
+					       (caar gnus-agent-buffer-alist))
+		      nil 'silent))
       (pop gnus-agent-buffer-alist))
     (while gnus-agent-group-alist
       (with-temp-file (caar gnus-agent-group-alist)
@@ -764,7 +770,8 @@ the actual number of articles toggled is returned."
  		       (gnus-uncompress-range
  			(cons (1+ (caar (last gnus-agent-article-alist)))
  			      (cdr (gnus-active group)))))
- 		    (gnus-list-of-unread-articles group))))
+ 		    (gnus-list-of-unread-articles group)))
+	(gnus-decode-encoded-word-function 'identity)) 
     ;; Fetch them.
     (when articles
       (gnus-message 7 "Fetching headers for %s..." group)
@@ -780,7 +787,9 @@ the actual number of articles toggled is returned."
  	    (gnus-agent-braid-nov group articles file))
  	  (gnus-make-directory (nnheader-translate-file-chars
  				(file-name-directory file)))
- 	  (write-region (point-min) (point-max) file nil 'silent)
+  	  (let ((coding-system-for-write
+  		 gnus-agent-file-coding-system))
+  	    (write-region (point-min) (point-max) file nil 'silent))
  	  (gnus-agent-save-alist group articles nil)
  	  (gnus-agent-enter-history
  	   "last-header-fetched-for-session"
@@ -894,6 +903,7 @@ the actual number of articles toggled is returned."
 (defun gnus-agent-fetch-group-1 (group method)
   "Fetch GROUP."
   (let ((gnus-command-method method)
+	(gnus-newsgroup-name group)
 	gnus-newsgroup-dependencies gnus-newsgroup-headers
 	gnus-newsgroup-scored gnus-headers gnus-score
 	gnus-use-cache articles arts
@@ -925,9 +935,10 @@ the actual number of articles toggled is returned."
 				     (gnus-agent-false nil))))))
 	;; No, we need to decide what we want.
 	(setq score-param
-	      (let ((score-method (or 
-				   (gnus-group-get-parameter group 'agent-score t)
-				   (caddr category))))
+	      (let ((score-method
+		     (or 
+		      (gnus-group-get-parameter group 'agent-score t)
+		      (caddr category))))
 		(when score-method
 		  (require 'gnus-score)
 		  (if (eq score-method 'file)
@@ -1073,7 +1084,7 @@ The following commands are available:
   (gnus-set-default-directory)
   (setq mode-line-process nil)
   (use-local-map gnus-category-mode-map)
-  (buffer-disable-undo (current-buffer))
+  (buffer-disable-undo)
   (setq truncate-lines t)
   (setq buffer-read-only t)
   (gnus-run-hooks 'gnus-category-mode-hook))
@@ -1191,7 +1202,7 @@ The following commands are available:
   (interactive "SCategory name: ")
   (when (assq category gnus-category-alist)
     (error "Category %s already exists" category))
-  (push (list category 'true nil nil)
+  (push (list category 'false nil nil)
 	gnus-category-alist)
   (gnus-category-write)
   (gnus-category-list))
@@ -1386,7 +1397,9 @@ The following commands are available:
 		 ;; Schedule the history line for nuking.
 		 (push (cdr elem) histories)))
 	     (gnus-make-directory (file-name-directory nov-file))
-	     (write-region (point-min) (point-max) nov-file nil 'silent)
+	     (let ((coding-system-for-write
+		    gnus-agent-file-coding-system))
+	       (write-region (point-min) (point-max) nov-file nil 'silent))
 	     ;; Delete the unwanted entries in the alist.
 	     (setq gnus-agent-article-alist
 		   (sort gnus-agent-article-alist 'car-less-than-car))
@@ -1411,7 +1424,7 @@ The following commands are available:
 	       (gnus-agent-save-alist group)
                ;; Mark all articles up to the first article
 	       ;; in `gnus-article-alist' as read.
-	       (when (caar gnus-agent-article-alist)
+	       (when (and info (caar gnus-agent-article-alist))
 		 (setcar (nthcdr 2 info)
 			 (gnus-range-add
 			  (nth 2 info)
@@ -1419,12 +1432,15 @@ The following commands are available:
 	       ;; Maybe everything has been expired from `gnus-article-alist'
 	       ;; and so the above marking as read could not be conducted,
 	       ;; or there are expired article within the range of the alist.
-	       (when (or (not (caar gnus-agent-article-alist))
-			 (> (car expired) (caar gnus-agent-article-alist)))  
-	       (setcar (nthcdr 2 info)
-		       (gnus-add-to-range
-			(nth 2 info)
-			(nreverse expired))))
+	       (when (and info
+			  expired
+			  (or (not (caar gnus-agent-article-alist))
+			      (> (car expired)
+				 (caar gnus-agent-article-alist))))
+		 (setcar (nthcdr 2 info)
+			 (gnus-add-to-range
+			  (nth 2 info)
+			  (nreverse expired))))
 	       (gnus-dribble-enter
 		(concat "(gnus-group-set-info '"
 			(gnus-prin1-to-string info)
