@@ -25,11 +25,13 @@
 ;;; Code:
 
 (eval-and-compile
-  (if (not (fboundp 'base64-encode-string))
+  (or (fboundp  'base64-decode-region)
       (require 'base64)))
+
 (require 'mm-util)
 (require 'rfc2047)
 (require 'qp)
+(require 'uudecode)
 
 (defun mm-encode-body ()
   "Encode a body.
@@ -60,9 +62,12 @@ If no encoding was done, nil is returned."
 		    (not (mm-coding-system-equal
 			  mime-charset buffer-file-coding-system)))
 	    (while (not (eobp))
-	      (if (eq (char-charset (following-char)) 'ascii)
+	      (if (eq (char-charset (char-after)) 'ascii)
 		  (when start
-		    (mm-encode-coding-region start (point) mime-charset)
+		    (save-restriction
+		      (narrow-to-region start (point))
+		      (mm-encode-coding-region start (point) mime-charset)
+		      (goto-char (point-max)))
 		    (setq start nil))
 		(unless start
 		  (setq start (point))))
@@ -90,22 +95,36 @@ If no encoding was done, nil is returned."
 ;;; Functions for decoding
 ;;;
 
-(defun mm-decode-content-transfer-encoding (encoding)
+(defun mm-decode-content-transfer-encoding (encoding &optional type)
   (cond
    ((eq encoding 'quoted-printable)
     (quoted-printable-decode-region (point-min) (point-max)))
    ((eq encoding 'base64)
-    (condition-case ()
-	(base64-decode-region (point-min) (point-max))
-      (error nil)))
+    (prog1
+	(condition-case ()
+	    (base64-decode-region (point-min) (point-max))
+	  (error nil))
+      (when (equal type "text/plain")
+	(goto-char (point-min))
+	(while (search-forward "\r\n" nil t)
+	  (replace-match "\n" t t)))))
    ((memq encoding '(7bit 8bit binary))
     )
    ((null encoding)
     )
+   ((eq encoding 'x-uuencode)
+    (condition-case ()
+	(uudecode-decode-region (point-min) (point-max))
+      (error nil)))
+   ((functionp encoding)
+    (condition-case ()
+	(funcall encoding (point-min) (point-max))
+      (error nil)))
    (t
-    (error "Can't decode encoding %s" encoding))))
+    (message "Unknown encoding %s; defaulting to 8bit" encoding)
+    )))
 
-(defun mm-decode-body (charset encoding)
+(defun mm-decode-body (charset &optional encoding)
   "Decode the current article that has been encoded with ENCODING.
 The characters in CHARSET should then be decoded."
   (setq charset (or charset rfc2047-default-charset))
@@ -117,9 +136,9 @@ The characters in CHARSET should then be decoded."
 	(when (and charset
 		   (setq mule-charset (mm-charset-to-coding-system charset))
 		   buffer-file-coding-system
-		   ;;(not (mm-coding-system-equal
-		   ;;	 buffer-file-coding-system mule-charset))
-		   )
+		   enable-multibyte-characters
+		   (or (not (eq mule-charset 'ascii))
+		       (setq mule-charset rfc2047-default-charset)))
 	  (mm-decode-coding-region (point-min) (point-max) mule-charset))))))
 
 (provide 'mm-bodies)
