@@ -78,12 +78,13 @@
    '((decode-coding-string . (lambda (s a) s))
      (encode-coding-string . (lambda (s a) s))
      (encode-coding-region . ignore)
-     (decode-coding-region . ignore)
      (coding-system-list . ignore)
+     (decode-coding-region . ignore)
      (char-int . identity)
      (device-type . ignore)
      (coding-system-equal . equal)
      (annotationp . ignore)
+     (set-buffer-file-coding-system . ignore)
      (make-char
       . (lambda (charset int)
 	  (int-to-char int)))
@@ -94,13 +95,18 @@
 	   prompt (mapcar (lambda (s) (list (symbol-name (car s))))
 			  mm-mime-mule-charset-alist)))))))
 
+(defvar mm-coding-system-list nil)
+(defun mm-get-coding-system-list ()
+  "Get the coding system list."
+  (or mm-coding-system-list
+      (setq mm-coding-system-list (mm-coding-system-list))))
+
 (defvar mm-charset-coding-system-alist
   (let ((rest
-	 '((us-ascii . iso-8859-1)
-	   (gb2312 . cn-gb-2312)
+	 '((gb2312 . cn-gb-2312)
 	   (iso-2022-jp-2 . iso-2022-7bit-ss2)
 	   (x-ctext . ctext)))
-	(systems (mm-coding-system-list))
+	(systems (mm-get-coding-system-list))
 	dest)
     (while rest
       (let ((pair (car rest)))
@@ -110,7 +116,6 @@
     dest)
   "Charset/coding system alist.")
 
-
 (defun mm-mule-charset-to-mime-charset (charset)
   "Return the MIME charset corresponding to MULE CHARSET."
   (let ((alist mm-mime-mule-charset-alist)
@@ -136,11 +141,14 @@ used as the line break code type of the coding system."
     (setq charset (intern (format "%s-%s" charset lbt))))
   (cond
    ;; Running in a non-MULE environment.
-   ((and (null (mm-coding-system-list))
+   ((and (null (mm-get-coding-system-list))
 	 (memq charset mm-known-charsets))
     charset)
+   ;; ascii
+   ((eq charset 'us-ascii)
+    'ascii)
    ;; Check to see whether we can handle this charset.
-   ((memq charset (mm-coding-system-list))
+   ((memq charset (mm-get-coding-system-list))
     charset)
    ;; Nope.
    (t
@@ -160,9 +168,15 @@ used as the line break code type of the coding system."
 
 (defsubst mm-enable-multibyte ()
   "Enable multibyte in the current buffer."
-  (when (fboundp 'set-buffer-multibyte)
+  (when (and (fboundp 'set-buffer-multibyte)
+	     (default-value enable-multibyte-characters))
     (set-buffer-multibyte t)))
 
+(defsubst mm-disable-multibyte ()
+  "Disable multibyte in the current buffer."
+  (when (fboundp 'set-buffer-multibyte)
+    (set-buffer-multibyte nil)))
+
 (defun mm-mime-charset (charset b e)
   (if (fboundp 'coding-system-get)
       (or
@@ -193,19 +207,21 @@ See also `with-temp-file' and `with-output-to-string'."
 	 (with-temp-buffer ,@forms)
        (let ((,multibyte (default-value enable-multibyte-characters))
 	     ,temp-buffer)
-	 (setq-default enable-multibyte-characters nil)
-	 (setq ,temp-buffer
-	       (get-buffer-create (generate-new-buffer-name " *temp*")))
 	 (unwind-protect
-	     (with-current-buffer ,temp-buffer
-	       ,@forms)
-	   (and (buffer-name ,temp-buffer)
-		(kill-buffer ,temp-buffer))
+	     (progn
+	       (setq-default enable-multibyte-characters nil)
+	       (setq ,temp-buffer
+		     (get-buffer-create (generate-new-buffer-name " *temp*")))
+	       (unwind-protect
+		   (with-current-buffer ,temp-buffer
+		     (let (buffer-file-coding-system)
+		       ,@forms))
+		 (and (buffer-name ,temp-buffer)
+		      (kill-buffer ,temp-buffer))))
 	   (setq-default enable-multibyte-characters ,multibyte))))))
 (put 'mm-with-unibyte-buffer 'lisp-indent-function 0)
 (put 'mm-with-unibyte-buffer 'edebug-form-spec '(body))
 
-
 (provide 'mm-util)
 
 ;;; mm-util.el ends here
