@@ -26,6 +26,7 @@
 (require 'mail-parse)
 (require 'mailcap)
 (require 'mm-bodies)
+(require 'mm-decode)
 
 ;;;
 ;;; Functions for displaying various formats inline
@@ -36,7 +37,9 @@
 	buffer-read-only image)
     (mm-with-unibyte-buffer
       (insert-buffer-substring (mm-handle-buffer handle))
-      (mm-decode-content-transfer-encoding (mm-handle-encoding handle))
+      (mm-decode-content-transfer-encoding
+       (mm-handle-encoding handle)
+       (car (mm-handle-type handle)))
       (setq image (make-image-specifier
 		   (vector (intern type) :data (buffer-string)))))
     (let ((annot (make-annotation image nil 'text)))
@@ -52,7 +55,9 @@
      ((equal type "plain")
       (with-temp-buffer
 	(insert-buffer-substring (mm-handle-buffer handle))
-	(mm-decode-content-transfer-encoding (mm-handle-encoding handle))
+	(mm-decode-content-transfer-encoding
+	 (mm-handle-encoding handle)
+	 (car (mm-handle-type handle)))
 	(setq text (buffer-string)))
       (let ((b (point)))
 	(insert text)
@@ -74,21 +79,47 @@
 	(w3-do-setup)
 	(mm-with-unibyte-buffer
 	  (insert-buffer-substring (mm-handle-buffer handle))
-	  (mm-decode-content-transfer-encoding (mm-handle-encoding handle))
+	  (mm-decode-content-transfer-encoding
+	   (mm-handle-encoding handle)
+	   (car (mm-handle-type handle)))
 	  (require 'url)
 	  (save-window-excursion
 	    (w3-region (point-min) (point-max))
 	    (setq text (buffer-string))))
-	(let ((b (point)))
-	  (insert text)
-	  (mm-handle-set-undisplayer
-	   handle
-	   `(lambda ()
-	      (let (buffer-read-only)
-		(delete-region ,(set-marker (make-marker) b)
-			       ,(set-marker (make-marker) (point)))))))))
-     )))
+	(mm-insert-inline handle text)))
+     ((or (equal type "enriched")
+	  (equal type "richtext"))
+      (save-excursion
+	(mm-with-unibyte-buffer
+	  (insert-buffer-substring (mm-handle-buffer handle))
+	  (mm-decode-content-transfer-encoding
+	   (mm-handle-encoding handle)
+	   (car (mm-handle-type handle)))
+	  (save-window-excursion
+	    (enriched-decode (point-min) (point-max))
+	    (setq text (buffer-string)))))
+      (mm-insert-inline handle text))
+     (t
+      (save-excursion
+	(mm-with-unibyte-buffer
+	  (insert-buffer-substring (mm-handle-buffer handle))
+	  (mm-decode-content-transfer-encoding
+	   (mm-handle-encoding handle)
+	   (car (mm-handle-type handle)))
+	  (setq text (buffer-string))))
+      (mm-insert-inline handle text)))))
 
+(defun mm-insert-inline (handle text)
+  "Insert TEXT inline from HANDLE."
+  (let ((b (point)))
+    (insert text)
+    (mm-handle-set-undisplayer
+     handle
+     `(lambda ()
+	(let (buffer-read-only)
+	  (delete-region ,(set-marker (make-marker) b)
+			 ,(set-marker (make-marker) (point))))))))
+  
 (defun mm-inline-audio (handle)
   (message "Not implemented"))
 
